from zerobouncesdk._zb_response import ZBResponse
import json


class ZBGetCreditsResponse(ZBResponse):
    credits: str = None

    def __init__(self, j):
        # super().__init__(j)
        _dict = json.loads(j)
        self.credits = _dict["Credits"]



from zerobouncesdk._zb_response import ZBResponse


class ZBGetApiUsageResponse(ZBResponse):
    total: int = 0
    status_valid: int = 0
    status_invalid: int = 0
    status_catch_all: int = 0
    status_do_not_mail: int = 0
    status_spamtrap: int = 0
    status_unknown: int = 0
    sub_status_toxic: int = 0
    sub_status_disposable: int = 0
    sub_status_role_based: int = 0
    sub_status_possible_trap: int = 0
    sub_status_global_suppression: int = 0
    sub_status_timeout_exceeded: int = 0
    sub_status_mail_server_temporary_error: int = 0
    sub_status_mail_server_did_not_respond: int = 0
    sub_status_greylisted: int = 0
    sub_status_antispam_system: int = 0
    sub_status_does_not_accept_mail: int = 0
    sub_status_exception_occurred: int = 0
    sub_status_failed_syntax_check: int = 0
    sub_status_mailbox_not_found: int = 0
    sub_status_unroutable_ip_address: int = 0
    sub_status_possible_typo: int = 0
    sub_status_no_dns_entries: int = 0
    sub_status_role_based_catch_all: int = 0
    sub_status_mailbox_quota_exceeded: int = 0
    sub_status_forcible_disconnect: int = 0
    sub_status_failed_smtp_connection: int = 0
    start_date: str = None
    end_date: str = None


from zerobouncesdk._zb_response import ZBResponse
from zerobouncesdk.zb_validate_status import ZBValidateStatus
from zerobouncesdk.zb_validate_sub_status import ZBValidateSubStatus


class ZBValidateResponse(ZBResponse):
    """This is the response for the validate API"""

    address: str = None
    """The email address you are validating."""

    status: ZBValidateStatus = None
    """one of [valid, invalid, catch-all, unknown, spamtrap, abuse, do_not_mail]"""

    sub_status: ZBValidateSubStatus = None
    """one of [antispam_system, greylisted, mail_server_temporary_error, forcible_disconnect, 
    mail_server_did_not_respond, timeout_exceeded, failed_smtp_connection, mailbox_quota_exceeded, 
    exception_occurred, possible_traps, role_based, global_suppression, mailbox_not_found, no_dns_entries, 
    failed_syntax_check, possible_typo, unroutable_ip_address, leading_period_removed, does_not_accept_mail, 
    alias_address, role_based_catch_all, disposable, toxic] """

    account: str = None
    """The portion of the email address before the "@" symbol."""

    domain: str = None
    """The portion of the email address after the "@" symbol."""

    did_you_mean: str = None
    """Suggestive Fix for an email typo"""

    domain_age_days: str = None
    """Age of the email domain in days or [null]."""

    free_email = False
    """[true/false] If the email comes from a free provider."""

    mx_found = False
    """[true/false] Does the domain have an MX record."""

    mx_record: str = None
    """ The preferred MX record of the domain"""

    smtp_provider: str = None
    """The SMTP Provider of the email or [null] [BETA]."""

    firstname: str = None
    """The first name of the owner of the email when available or [null]."""

    lastname: str = None
    """The last name of the owner of the email when available or [null]."""

    gender: str = None
    """The gender of the owner of the email when available or [null]."""

    city: str = None
    """The city of the IP passed in."""

    region: str = None
    """The region/state of the IP passed in."""

    zipcode: str = None
    """The zipcode of the IP passed in."""

    country: str = None
    """The country of the IP passed in."""

    processed_at: str = None
    """The UTC time the email was validated."""

    error: str = None

    def __init__(self, j):
        super().__init__(j)
        self.status = None if self.status is None else ZBValidateStatus(self.status)
        self.sub_status = None if self.sub_status is None else ZBValidateSubStatus(self.sub_status)






import json
import ntpath
import urllib.parse
import urllib.request
from datetime import date
import os
import requests
from requests_toolbelt.multipart.encoder import MultipartEncoder

from zerobouncesdk import \
    ZBValidateResponse, ZBApiException, ZBMissingApiKeyException, \
    ZBSendFileResponse, ZBGetCreditsResponse, ZBFileStatusResponse, \
    ZBDeleteFileResponse, ZBGetApiUsageResponse, ZBGetFileResponse


__apiBaseUrl = "https://api.zerobounce.net/v2"
__bulkApiBaseUrl = "https://bulkapi.zerobounce.net/v2"
__api_key = None

__log_enabled: bool = False


def initialize(api_key):
    global __api_key
    __api_key = api_key


def set_log_enabled(log_enabled: bool):
    global __log_enabled
    __log_enabled = log_enabled


def validate(email: str, ip_address: str = None) -> ZBValidateResponse:
    """
    Parameters
    ----------
    email: str
        The email address you want to validate
    ip_address: str on None
        The IP Address the email signed up from (Can be blank)

    Raises
    ------
    ZBApiException
    ZBMissingApiKeyException

    Returns
    -------
    response: ZBValidateResponse
        Returns a ZBValidateResponse object if the request was successful
    """

    __check_api_key()

    return __request(
        __apiBaseUrl + "/validate?api_key=" + __api_key
        + "&email=" + email
        + "&ip_address=" + ("" if ip_address is None else ip_address),
        ZBValidateResponse)


def get_credits() -> ZBGetCreditsResponse:
    """This API will tell you how many credits you have left on your account. It's simple, fast and easy to use.

    Raises
    ------
    ZBApiException
    ZBMissingApiKeyException

    Returns
    -------
    response: ZBGetCreditsResponse
        Returns a ZBGetCreditsResponse object if the request was successful
    """

    __check_api_key()

    return __request(
        __apiBaseUrl + "/getcredits?api_key=" + __api_key,
        ZBGetCreditsResponse)


def send_file(
        file_path: str, email_address_column: int,
        return_url: str = None,
        first_name_column: int = None, last_name_column: int = None,
        gender_column: int = None, ip_address_column: int = None,
        has_header_row: bool = False) -> ZBSendFileResponse:
    """
    The sendfile API allows user to send a file for bulk email validation

    Raises
    ------
    ZBApiException
    ZBMissingApiKeyException

    Returns
    -------
    response: ZBSendFileResponse
        Returns a ZBSendFileResponse object if the request was successful
    """

    __check_api_key()

    fields = {
        # a file upload field
        'file': (ntpath.basename(file_path), open(file_path, 'rb')),
        # plain text fields
        'api_key': __api_key,
        'email_address_column': repr(email_address_column),
    }

    if return_url is not None:
        fields.update({'return_url': return_url})
    if first_name_column is not None:
        fields.update({'first_name_column': repr(first_name_column)})
    if last_name_column is not None:
        fields.update({'last_name_column': repr(last_name_column)})
    if gender_column is not None:
        fields.update({'gender_column': repr(gender_column)})
    if ip_address_column is not None:
        fields.update({'ip_address_column': repr(ip_address_column)})
    if has_header_row is not None:
        fields.update({'has_header_row': "true" if has_header_row else "false"})

    multipart_data = MultipartEncoder(fields=fields)

    response = requests.post(
        url=__bulkApiBaseUrl + "/sendfile",
        data=multipart_data,
        # files=files,
        headers={"Content-Type": multipart_data.content_type}
    )

    response_json = json.dumps(response.json())

    if __log_enabled:
        print("ZeroBounce response json: " + response_json)

    response.raise_for_status()

    response_object = ZBSendFileResponse(response_json)
    return response_object


def file_status(file_id: str):
    """

    Parameters
    ----------
    file_id: str
        The returned file ID when calling sendFile API.

    Raises
    ------
    ZBApiException
    ZBMissingApiKeyException

    Returns
    -------
    response: ZBSendFileResponse
        Returns a ZBSendFileResponse object if the request was successful
    """

    __check_api_key()

    return __request(
        __bulkApiBaseUrl + "/filestatus?api_key=" + __api_key + "&file_id=" + file_id,
        ZBFileStatusResponse)


def delete_file(file_id: str) -> ZBDeleteFileResponse:
    """
    Parameters
    ----------
    file_id: str
        The returned file ID when calling sendFile API.

    Raises
    ------
    ZBApiException
    ZBMissingApiKeyException

    Returns
    -------
    response: ZBDeleteFileResponse
        Returns a ZBDeleteFileResponse object if the request was successful
    """
    __check_api_key()

    return __request(
        __bulkApiBaseUrl + "/deletefile?api_key=" + __api_key + "&file_id=" + file_id,
        ZBDeleteFileResponse)


def get_api_usage(start_date: date, end_date: date) -> ZBGetApiUsageResponse:
    """
    Parameters
    ----------
    start_date: date
        The start date of when you want to view API usage
    end_date: date
        The end date of when you want to view API usage

    Raises
    ------
    ZBApiException
    ZBMissingApiKeyException

    Returns
    -------
    response: ZBGetApiUsageResponse
        Returns a ZBGetApiUsageResponse object if the request was successful
    """
    __check_api_key()

    return __request(
        __apiBaseUrl + "/getapiusage?api_key=" + __api_key +
        "&start_date=" + start_date.strftime('%Y-%m-%d') +
        "&end_date=" + end_date.strftime('%Y-%m-%d'),
        ZBGetApiUsageResponse)


def get_file(file_id: str, local_download_path: str) -> ZBGetFileResponse:
    """
    The getfile API allows users to get the validation results file for the file been submitted using sendfile API

    Parameters
    ----------
    file_id: str
        The returned file ID when calling sendFile API.
    local_download_path: str
        The local path where the file will be downloaded.

    Returns
    -------
    response: ZBGetFileResponse
        Returns a ZBGetFileResponse object if the request was successful
    """
    __check_api_key()

    try:
        os.makedirs(ntpath.dirname(local_download_path), exist_ok=True)
        url = __bulkApiBaseUrl + "/getFile?api_key=" + __api_key + "&file_id=" + file_id
        if __log_enabled:
            print("ZeroBounce request url=" + url)
        urllib.request.urlretrieve(url, local_download_path)

        response = ZBGetFileResponse()
        response.localFilePath = local_download_path
        return response
    except Exception as e:
        raise ZBApiException(str(e))


def __check_api_key():
    if __api_key is None:
        raise ZBMissingApiKeyException(
            "ZeroBounce SDK is not initialized. "
            "Please call zerobouncesdk.initialize(<apiKey>) first")


def __request(url, response_class):
    try:
        if __log_enabled:
            print("ZeroBounce request url=" + url)
        connection = urllib.request.urlopen(url)
        response = connection.read().decode('utf-8')
        if __log_enabled:
            print("ZeroBounce response string: " + response)
        response_object = response_class(response)
        return response_object
    except TypeError as e:
        raise ZBApiException(str(e))
    except Exception as e:
        raise ZBApiException(str(e))


def __pretty_print_post(req):
    print('{}\n{}\n{}\n\n{}'.format(
        '-----------START-----------',
        req.method + ' ' + req.url,
        '\n'.join('{}: {}'.format(k, v) for k, v in req.headers.items()),
        str(req.body).replace("\\r\\n", "\n"),
    ))



from enum import Enum


class ZBValidateSubStatus(Enum):
    none = ""
    antispam_system = "antispam_system"
    greylisted = "greylisted"
    mail_server_temporary_error = "mail_server_temporary_error"
    forcible_disconnect = "forcible_disconnect"
    mail_server_did_not_respond = "mail_server_did_not_respond"
    timeout_exceeded = "timeout_exceeded"
    failed_smtp_connection = "failed_smtp_connection"
    mailbox_quota_exceeded = "mailbox_quota_exceeded"
    exception_occurred = "exception_occurred"
    possible_traps = "possible_traps"
    role_based = "role_based"
    global_suppression = "global_suppression"
    mailbox_not_found = "mailbox_not_found"
    no_dns_entries = "no_dns_entries"
    failed_syntax_check = "failed_syntax_check"
    possible_typo = "possible_typo"
    unroutable_ip_address = "unroutable_ip_address"
    leading_period_removed = "leading_period_removed"
    does_not_accept_mail = "does_not_accept_mail"
    alias_address = "alias_address"
    role_based_catch_all = "role_based_catch_all"
    disposable = "disposable"
    toxic = "toxic"



from enum import Enum


class ZBValidateStatus(Enum):
    valid = "valid"
    invalid = "invalid"
    catch_all = "catch-all"
    unknown = "unknown"
    spamtrap = "spamtrap"
    abuse = "abuse"
    do_not_mail = "do_not_mail"



from zerobouncesdk.zb_exceptions import ZBApiException, ZBMissingApiKeyException
from zerobouncesdk.zb_validate_response import ZBValidateResponse
from zerobouncesdk.zb_validate_status import ZBValidateStatus
from zerobouncesdk.zb_send_file_response import ZBSendFileResponse
from zerobouncesdk.zb_get_credits_response import ZBGetCreditsResponse
from zerobouncesdk.zb_file_status_response import ZBFileStatusResponse
from zerobouncesdk.zb_delete_file_response import ZBDeleteFileResponse
from zerobouncesdk.zb_get_api_usage_response import ZBGetApiUsageResponse
from zerobouncesdk.zb_get_file_response import ZBGetFileResponse



import json


class ZBResponse(object):
    def __init__(self, j=None):
        if j is not None:
            self.__dict__ = json.loads(j)

    def __str__(self) -> str:
        return str(self.__class__.__name__) + "=" + str(self.__dict__)



from setuptools import setup

setup(
    name='zerobouncesdk',
    version='0.0.3',
    description='The ZeroBounce SDK for Python programming language',
    license='MIT',
    packages=['zerobouncesdk'],
    author='ZeroBounce',
    author_email='integrations@zerobounce.net',
    keywords=['zero', 'bounce', 'sdk'],
    url='https://github.com/zerobounce-llc/zero-bounce-python-sdk', install_requires=['requests', 'requests_toolbelt']
)
